# -*- coding: utf-8 -*-
# encoding: utf-8

# Gravaty
# Copyright © 2013  Marco Bresciani
#
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not, see
# <http://www.gnu.org/licenses/>.

require 'digest/md5'
require 'r18n-desktop'

# This class is a simple APi to retrieve an URL with specified options from Gravatar site.
#
# The only needed parameter to create a Gravaty object is the reference email address.
class Gravaty
  include R18n::Helpers
  attr_reader :email, :digest

  # The possible built in options to retrieve special avatars sizes range for the avatar image size.
  DEFAULT_OPTIONS = ['404' 'mm' 'identicon' 'monsterid' 'wavatar' 'retro' 'blank']

  ##
  # Gravaty.new(str)    -> new_gravaty
  #
  # Creates a new Gravaty described by the user's email.
  #
  # Usage:
  #   a_gravaty = Gravaty.new user@example.com
  #
  # parameter email [String] is the email address to retrieve Gravatar data for.
  def initialize(email)
    @email = email
    @digest = Digest::MD5.hexdigest @email.downcase
    R18n.from_env(Pathname(__FILE__).dirname.expand_path.to_s + '/i18n/')
  end

  # a_gravaty.image_request                                                 -> String
  # a_gravaty.image_request { pixel_size: 42 }                              -> String
  # a_gravaty.image_request { pixel_size: 42, type: 'png' }                 -> String
  # a_gravaty.image_request { secure: false }                               -> String
  # a_gravaty.image_request { type: 'jpg', force: true }                    -> String
  # a_gravaty.image_request { pixel_size: 42, secure: true, type: 'png' }   -> String
  #
  #
  # The main method to ask for a specific Gravatar URL.
  #
  # parameter type [String] is the possibly desired specific image format. Valid formats are JP(E)G, PNG or GIF.
  # parameter pixel_size [Integer] it the size in pixel of the image. From 1 to 2048.
  # parameter force [TrueClass || FalseClass] is a boolean to specify if the default has to be forced when no image is found.
  # param secure [TrueClass || FalseClass] is a boolean to specify whether the resulting URL shall be HTTPS or HTTP type.
  #
  # return [String] The Gravatar site URL with specified options and configuration parameters.
  def image_request(args = {})
    raise ArgumentError, t.error.hash if args.nil?
    type = args[:type] || nil
    pixel_size = args[:pixel_size] || nil
    force = args[:force]
    secure = (args[:secure].nil?() ? true : args[:secure])
    image_param_request type, pixel_size, force, secure
  end

  # a_gravaty.image_request                         -> String
  # a_gravaty.image_request nil, nil, nil, false    -> String
  # a_gravaty.image_request nil, 640                -> String
  # a_gravaty.image_request 'png'                   -> String
  # a_gravaty.image_request 'jpg', nil, true        -> String
  # a_gravaty.image_request 'png', 42, true         -> String
  #
  # See image_request but parameters shall be passed one by one. Can be used when the list is well know and fixed.
  #
  # parameter type [String] is the possibly desired specific image format. Valid formats are JP(E)G, PNG or GIF.
  # parameter pixel_size [Integer] it the size in pixel of the image. From 1 to 2048.
  # parameter force [TrueClass || FalseClass] is a boolean to specify if the default has to be forced when no image is found.
  # param secure [TrueClass || FalseClass] is a boolean to specify whether the resulting URL shall be HTTPS or HTTP type.
  #
  # return [String] The Gravatar site URL with specified options and configuration parameters.
  def image_param_request(type = nil, pixel_size = nil, force = false, secure = true)
    image_url = @digest + extension(type)
    full_url = base_url(secure) + image_url
    full_url + parameters(pixel_size, force)
  end

  def extension(type = nil)
    extension = ''
    case type.to_s.downcase
      when 'jpg', 'jpeg', 'png', 'gif'
        extension = '.' + type.downcase
      else
        raise ArgumentError, t.error.type(type.to_s)
    end if not type.nil?
    extension
  end

  def base_url(secure = false)
    suffix = '.gravatar.com/avatar/'

    return 'http://www' + suffix if not secure
    'https://secure' + suffix
  end

  def parameters(pixel_size = nil, force = false)
    parameters = ''
    parameters += size(pixel_size)

    force_param = force_default force
    unless force_param.empty?
      parameters += '&' + force_param
    end

    return ('?' + parameters) if not parameters.empty?
    parameters
  end

  def size(pixel_size = nil)
    return '' if pixel_size.nil?
    raise ArgumentError, t.error.value(pixel_size) if not (1...2048).include? pixel_size.to_i
    's=' + pixel_size.to_s if not pixel_size.nil?
  end

  def force_default(force = false)
    return 'f=y' if force
    ''
  end

  #def image_request(type = nil, pixel_size = nil, force_default = false, secure = false)
  #  extension = extension(type)
  #
  #  image_url = @digest + extension + size(pixel_size) + default(default_image) + force_default(force_default)
  #
  #  return BASIC_URL + image_url if not secure
  #  SECURE_URL + image_url
  #end
  #
end
